%%%%%%%%%%%%%%
% Exercise 1 %
%%%%%%%%%%%%%%

clc, clear, close all

% ------------------------ %
%        INPUT DATA        %
% ------------------------ %

T=2*pi;                                         % signal length
n=2048;                                         % number of samples
t=linspace(0,T,n);                              % Sampling instants
ni_1=3;   ni_2=10;                              % Frequency of the two sinusoidal segments      
C=0.5;                                          % Standard deviation of Gabor Transform kernel

% -------------------------------------- %
% Generating signal and Fourier spectrum %
% -------------------------------------- %
x=sin(2*pi*ni_1*t).*(t>1 & t<3)+0.5*sin(2*pi*ni_2*t).*(t>4 & t<5);

k=fftshift((2*pi/T)*[0:(n/2-1) (-n/2):-1]);     % Wavenumbers vector
S=fftshift(fft(x));                             % Fourier spectrum

figure(1)
subplot(2,1,1)
plot(t,x,'k','linewidth',2)
xlabel('$t$','interpreter','latex','Fontsize',12)
ylabel('$x$','interpreter','latex','Fontsize',12)
set(gca,'TickLabelInterpreter','latex','Fontsize',10)
xlim([0 T])
subplot(2,1,2)
loglog(k./(2*pi),abs(S),'k','linewidth',2)
xlabel('$\nu$','interpreter','latex','Fontsize',12)
ylabel('$\hat{S}$','interpreter','latex','Fontsize',12)
xlim([1 max(k)./(2*pi)])
ylim([10^-3 10^3])
set(gca,'TickLabelInterpreter','latex','Fontsize',10)

% ----------------------%
% Computing Spectrogram %
% ----------------------%

tv=t(1:1:numel(t));                             % Time lattice
SG=zeros(numel(k),numel(tv));                   % Allocating memory
for i=1:numel(tv)       
    g=1./(C*sqrt(2*pi)).*exp(-0.5*(((t-tv(i))/(C)).^2));
    s=abs(fftshift(fft(x.*g)))./sum(g);
    SG(:,i)=s(:);
end

figure(2)
pcolor(tv,k./(2*pi),SG)
ylim([1 100])
colormap jet(16)
shading interp
set(gca,'YScale','log')
xlabel('$t$','interpreter','latex','Fontsize',12)
ylabel('$\nu$','interpreter','latex','Fontsize',12)
set(gca,'TickLabelInterpreter','latex','Fontsize',10)
set(gcf,'units','centimeters','position',[1 1 16 9])
s1=sprintf('%1.1f',C);
s=strcat('$\alpha = ',s1,'$');
h=title(s,'interpreter','latex');
set(h,'fontsize',12,'fontweight','bold')


