%%%%%%%%%%%%%%
% Exercise 3 %
%%%%%%%%%%%%%%

% Time-frequency analysis of hot-wire data in CICLOPE

clc, clear, close all

% ------------------------ %
%        INPUT DATA        %
% ------------------------ %
load('CASE1_HW.mat')
C=0.1;                           % Standard deviation of Gabor Transform kernel
t0=10;                           % Spacing of the temporal lattice [s]
fs=10000;                        % Sampling Frequency [Hz]
dwn=10;                          % Downsampling factor

% ------------------------------- %
% Building signal and time vector %
% ------------------------------- %
x=ut(10001:round(dwn):40000,1)'; % Downsampling signal at 1kHz
x=x-mean(x);                     % removing mean
n=numel(x);
t=(0:n-1)/fs*round(dwn);         % building time vector [s]
T=t(end);                        % Time length of the signal

% -------------------------- %
% Computing Fourier spectrum %
% -------------------------- %
k=fftshift((2*pi/T)*[0:(n/2-1) (-n/2):-1]);     % Wavenumbers vector
S=fftshift(fft(x));                             % Fourier spectrum

figure(1)
subplot(2,1,1)
plot(t,x,'k','linewidth',2)
xlabel('$t [s]$','interpreter','latex','Fontsize',12)
ylabel('$u [m/s]$','interpreter','latex','Fontsize',12)
set(gca,'TickLabelInterpreter','latex','Fontsize',10)
subplot(2,1,2)
loglog(k./(2*pi),abs(S)./max(abs(S)),'k','linewidth',2)
xlabel('$\nu [Hz]$','interpreter','latex','Fontsize',12)
ylabel('$\hat{S}$','interpreter','latex','Fontsize',12)
xlim([1 max(k)./(2*pi)])
set(gca,'TickLabelInterpreter','latex','Fontsize',10)

%%%%%%%%%%%%%%%%%%%%%

tau=T/2;
g=1./(C*sqrt(2*pi)).*exp(-0.5*(((t-tau)/(C)).^2));

figure(2)
subplot(3,1,1)
plot(t,x,'Linewidth',1.5); hold on; plot(t,g,'k')
subplot(3,1,2)
plot(t,x.*g,'Linewidth',1.5);
subplot(3,1,3)
plot(k./(2*pi),abs(fftshift(fft(x.*g))),'Linewidth',1.5)


% ----------------------%
% Computing Spectrogram %
% ----------------------%

tv=t(1:10:numel(t));
SG=zeros(numel(k),numel(tv));
for i=1:numel(tv)
    g=1./(C*sqrt(2*pi)).*exp(-0.5*(((t-tv(i))/(C)).^2));
    s=abs(fftshift(fft(x.*g)))./sum(g);
    SG(:,i)=s(:);
end

figure(3)
pcolor(tv,k./(2*pi),SG)
ylim([1 100])
colormap jet(16)
shading interp
set(gca,'YScale','log')
xlabel('$t [s]$','interpreter','latex','Fontsize',12)
ylabel('$\nu [Hz]$','interpreter','latex','Fontsize',12)
set(gca,'TickLabelInterpreter','latex','Fontsize',10,'XTick',0.5:0.5:3)
caxis([0.01 0.5])

%%%%%%%%%%%%%%%%%%%
h=figure(4);
[wt,f,coi]=cwt(x,fs/round(dwn));
pcolor(t,f,abs(wt))
shading interp
hold on
plot(t,coi,'w--','Linewidth',2)
colormap jet(16)
set(gca,'YScale','log')
A1 = area(t,coi);
A1.EdgeColor = 'none';
A1.FaceColor = [0.5 0.5 0.5];
alpha(A1,0.8);
ylim([1 100])
caxis([0.01 0.5])
xlabel('$t [s]$','interpreter','latex','Fontsize',12)
ylabel('$\nu [Hz]$','interpreter','latex','Fontsize',12)
set(gca,'TickLabelInterpreter','latex','Fontsize',10,'XTick',0.5:0.5:3)

