%%%%%%%%%%%%%%
% Exercise 4 %
%%%%%%%%%%%%%%

% DWT of fluid data

clc, clear, close all

% ------------------------ %
%        INPUT DATA        %
% ------------------------ %
load('.\GridStruc')         % Grid points of the simulation
load('.\pinball_000001')    % Instantaneous velocity field
noise_level=0.05;           % Input noise level

% Contaminating data with noise
Uex=U; 
U=U+noise_level.*randn(size(U));

% ------------------------------%
% 2-Level wavelet decomposition %
% ------------------------------%
[C,L]=wavedec2(U,2,'db1');

A1=appcoef2(C,L,'db1',1);           % Approximation coefficients at level 1
[H1 V1 D1]=detcoef2('a',C,L,1);     % Detail coefficients at level 1
A2=appcoef2(C,L,'db1',2);           % Approximation coefficients at level 2
[H2 V2 D2]=detcoef2('a',C,L,2);     % Detail coefficients at level 2

% Rescaling images to 8-bits
A1=wcodemat(A1,255); H1=wcodemat(H1,255); V1=wcodemat(V1,255); D1=wcodemat(D1,255);
A2=wcodemat(A2,255); H2=wcodemat(H2,255); V2=wcodemat(V2,255); D2=wcodemat(D2,255);

lev2=[A2 H2; V2 D2];
lev1=[lev2 H1; V1 D1];


figure
imagesc(lev1)
colormap jet(16)
axis equal
xlim([1 1000])
ylim([1 500])
set(gca,'TickLabelInterpreter','latex','fontsize',10)


% ------------------------%
% Filtering with wavelets %
% ------------------------%
[C,L]=wavedec2(U,4,'db4');         % DWT with Daubechies wavelet of 4th order
C_sorted=sort(C,'descend');
retain=0.02;                        % Fraction of coefficients to reatin
nk=round(retain*numel(C));
threshold=abs(C_sorted(nk+1));      % Selecting threshold
C_clean = wthresh(C,'s',threshold); % Soft thresholding

Urec=waverec2(C_clean,L,'db4');     % Reconstruction

% -----------------------%
% Filtering with Fourier %
% -----------------------%
S=fftshift(fft2(U));
[nx,ny]=size(U);
[sx,sy]=meshgrid(-ny/2+1:ny/2,-nx/2+1:nx/2);
I=((sx.^2+sy.^2)<(60^2));
S_clean=S.*I;
Urec_F=real(ifft2(ifftshift(S_clean)));


figure
subplot(2,2,1)
pcolor(X,Y,Uex)
title('Original Signal','fontsize',12,'interpreter','latex')
subplot(2,2,2)
pcolor(X,Y,U)
title('Noisy Signal','fontsize',12,'interpreter','latex')
subplot(2,2,3)
pcolor(X,Y,Urec)
title('Filtered DWT','fontsize',12,'interpreter','latex')
subplot(2,2,4)
pcolor(X,Y,Urec_F)
title('Filtered FFT','fontsize',12,'interpreter','latex')
for i=1:4
    subplot(2,2,i)
    axis equal
    colormap jet(16)
    caxis([-0.5 1.5])
    shading interp
    xlim([-5 15])
    ylim([-5 5])
    hold on
    rectangle('Position',[-(3/2)*cosd(30)-0.5 -0.5 1 1],'Curvature',[1,1],'EdgeColor','k','FaceColor','w');
    rectangle('Position',[-0.5 -(3/4)-0.5 1 1],'Curvature',[1,1],'EdgeColor','k','FaceColor','w');
    rectangle('Position',[-0.5 (3/4)-0.5 1 1],'Curvature',[1,1],'EdgeColor','k','FaceColor','w');
    set(gca,'TickLabelInterpreter','latex','fontsize',10)
    xlabel('$X/D$','interpreter','latex','fontsize',12)
    ylabel('$Y/D$','interpreter','latex','fontsize',12)
end